@@ -245,6 +245,13 @@ function recursive_unwrap(x::AbstractArray)
     symbolic_type(x) == ArraySymbolic() ? unwrap(x) : recursive_unwrap.(x)
 end
 
+function recursive_unwrap(x::SparseMatrixCSC)
+    I, J, V = findnz(x)
+    V = recursive_unwrap(V)
+    m, n = size(x)
+    return sparse(I, J, V, m, n)
+end
+
 recursive_unwrap(x) = unwrap(x)
 
 function recursive_unwrap(x::AbstractDict)
@@ -773,7 +780,7 @@ function get_mtkparameters_reconstructor(srcsys::AbstractSystem, dstsys::Abstrac
     # `syms[1]` is always the tunables because `srcsys` will have initials.
     tunable_syms = syms[1]
     tunable_getter = if isempty(tunable_syms)
-        Returns(SizedVector{0, Float64}())
+        Returns(SVector{0, Float64}())
     else
         p_constructor ∘ concrete_getu(srcsys, tunable_syms; eval_expression, eval_module)
     end
@@ -796,7 +803,7 @@ function get_mtkparameters_reconstructor(srcsys::AbstractSystem, dstsys::Abstrac
         end
         p_constructor ∘ concrete_getu(srcsys, initsyms; eval_expression, eval_module)
     else
-        Returns(SizedVector{0, Float64}())
+        Returns(SVector{0, Float64}())
     end
     discs_getter = if isempty(syms[3])
         Returns(())
@@ -916,7 +923,7 @@ function (rip::ReconstructInitializeprob)(srcvalp, dstvalp)
     if newp isa MTKParameters
         # and initials portion
         buf, repack, alias = SciMLStructures.canonicalize(SciMLStructures.Initials(), newp)
-        if eltype(buf) != T
+        if eltype(buf) != T && !(buf isa SVector{0})
             newbuf = similar(buf, T)
             copyto!(newbuf, buf)
             newp = repack(newbuf)
@@ -1141,8 +1148,10 @@ function maybe_build_initialization_problem(
     if is_split(sys)
         buffer, repack, _ = SciMLStructures.canonicalize(SciMLStructures.Tunable(), initp)
         initp = repack(floatT.(buffer))
-        buffer, repack, _ = SciMLStructures.canonicalize(SciMLStructures.Initials(), initp)
-        initp = repack(floatT.(buffer))
+        if !(initp.initials isa StaticVector{0})
+            buffer, repack, _ = SciMLStructures.canonicalize(SciMLStructures.Initials(), initp)
+            initp = repack(floatT.(buffer))
+        end
     elseif initp isa AbstractArray
         if ArrayInterface.ismutable(initp)
             initp′ = similar(initp, floatT)
